#include "SystemState.h"
#include <EEPROM.h>
#include "EepromUtil.h"
#include "debugWeb.h"
#include "sensors.h"

extern AsyncWebSocket wsd;

SystemState::SystemState() {
	readSettings();
}

void SystemState::readSettings(){
	DEBUG_PRINTLN(G("Read settings"));
	char result[5];

	EEPROM.begin(512);
	eepromUtil.eeprom_read_string(E_INIT, result, 5);
	if (strcmp(result, "INIT") != 0) {
		//EEPROM not initialised so set default values
		DEBUG_PRINTLN(G("Set EEPROM values to default"));
		resetParameters();
	}

	Defrost.defrostTemperature = eepromUtil.eeprom_read_byte(E_DEFROSTTEMP) - DEFROST_OFFSET;			//Temperature to run defrost cycle
	Defrost.defrostRuntime = eepromUtil.eeprom_read_word(E_DEFROSTTIME);								//Defrost cycle time (minutes)
	Defrost.defrostUpperTemperature = eepromUtil.eeprom_read_byte(E_DEFROSTMAXTEMP);					//Defrost Maximum temperature (temperature to stop defrost, overrides timer)
	Defrost.defrostDelay = eepromUtil.eeprom_read_word(E_DEFROSTDELAY);
	Defrost.defrostTimeBetween = eepromUtil.eeprom_read_byte(E_TIMEBBETWEENDEFROSTS);
	maxCompTemperature = eepromUtil.eeprom_read_byte(E_MAXCOMPTEMP);									//Maximum compressor temperature for safety
	heatT = eepromUtil.eeprom_read_byte(E_HEATTEMP);													//Fixed heating temperature
	coolT = eepromUtil.eeprom_read_byte(E_COOLTEMP);													//Fixed cooling temperature
	hysterisis = eepromUtil.eeprom_read_byte(E_HYSTERISIS);												//Hysterisis delay
	operatingMode = eepromUtil.eeprom_read_byte(E_OPERATINGMODE);										//Operating mode
	setRoomTemperature = eepromUtil.eeprom_read_byte(E_ROOMTEMP);										//Desired room temperature
	masterOnOff = eepromUtil.eeprom_read_byte(E_ONOFF);
	useThermostat = eepromUtil.eeprom_read_byte(E_USETHERMOSTAT);
	thermostatID = eepromUtil.eeprom_read_int(E_THERMOSTATID);
	eepromUtil.eeprom_read_string(E_USER, www_username, sizeof(www_username));
	eepromUtil.eeprom_read_string(E_PASS, www_password, sizeof(www_password));
	eepromUtil.eeprom_read_string(E_HOSTNAME, ss.hostName, sizeof(ss.hostName));
	Serial.printf("Hostname:%s\n", ss.hostName);
	startTime = eepromUtil.eeprom_read_byte(E_STARTTIME);
	stopTime = eepromUtil.eeprom_read_byte(E_STOPTIME);
	automatic = eepromUtil.eeprom_read_byte(E_AUTOMATIC);
	adcOffset = eepromUtil.eeprom_read_byte(E_ADCOFFSET);
	nightTemperature = eepromUtil.eeprom_read_byte(E_NIGHTREDUCTION);
	nightHour = eepromUtil.eeprom_read_byte(E_NIGHTHOUR);
	dayHour = eepromUtil.eeprom_read_byte(E_DAYHOUR);
	bUseWeather = eepromUtil.eeprom_read_byte(E_USEWEATHER);
	bRunPumpWhenOff = eepromUtil.eeprom_read_byte(E_RUNPUMPWHENOFF);
	eepromUtil.eeprom_read_string(E_TOPIC, ss.topic, 16, true);
	snprintf(ss.topicr, sizeof(ss.topicr), "%sr", ss.topic);

	currentOperatingState = eepromUtil.eeprom_read_byte(E_LASTSYSTEMSTATE);									//Last operating state. If running then we need a delay on startup

	if (currentOperatingState == ON)
	{
		startupDelay = true;
		setOperatingState(OFF);
		DEBUG_PRINTLN(G("StartupDelay=TRUE"));
	}
	thermostatIP() = thermostatIP();			//Get IP address of thermostat
	debugOutParameters();

}

//Reset all parameters to default values
void SystemState::resetParameters() {
	EEPROM.begin(512);
	eepromUtil.eeprom_erase_all(0x00);
	eepromUtil.eeprom_write_word(E_DEFROSTDELAY, 300);
	eepromUtil.eeprom_write_byte(E_DEFROSTTIME, 5);
	eepromUtil.eeprom_write_byte(E_DEFROSTTEMP, -6 + DEFROST_OFFSET);
	eepromUtil.eeprom_write_byte(E_DEFROSTMAXTEMP, 10);
	eepromUtil.eeprom_write_byte(E_MAXCOMPTEMP, 110);
	eepromUtil.eeprom_write_byte(E_HEATTEMP, 34);
	eepromUtil.eeprom_write_byte(E_COOLTEMP, 15);
	eepromUtil.eeprom_write_byte(E_HYSTERISIS, 5);
	eepromUtil.eeprom_write_byte(E_ROOMTEMP, 21);
	eepromUtil.eeprom_write_byte(E_OPERATINGMODE, HEAT);
	eepromUtil.eeprom_write_byte(E_LASTSYSTEMSTATE, OFF);
	eepromUtil.eeprom_write_byte(E_ONOFF, OFF);
	eepromUtil.eeprom_write_byte(E_IP0, 0);
	eepromUtil.eeprom_write_byte(E_IP1, 0);
	eepromUtil.eeprom_write_byte(E_IP2, 0);
	eepromUtil.eeprom_write_byte(E_IP3, 0);
	eepromUtil.eeprom_write_byte(E_USETHERMOSTAT, false);
	eepromUtil.eeprom_write_byte(E_THERMOSTATID, 0);
	eepromUtil.eeprom_write_string(E_USER, "admin");
	eepromUtil.eeprom_write_string(E_PASS, "admin");
	eepromUtil.eeprom_write_byte(E_STARTTIME, 23);
	eepromUtil.eeprom_write_byte(E_STOPTIME, 12);
	eepromUtil.eeprom_write_byte(E_AUTOMATIC, false);
	eepromUtil.eeprom_write_byte(E_ADCOFFSET, 0);
	eepromUtil.eeprom_write_byte(E_NIGHTREDUCTION, 4);
	eepromUtil.eeprom_write_byte(E_NIGHTHOUR, 22);
	eepromUtil.eeprom_write_byte(E_DAYHOUR, 6);
	eepromUtil.eeprom_write_byte(E_USEWEATHER, true);
	eepromUtil.eeprom_write_byte(E_RUNPUMPWHENOFF, true);

	eepromUtil.eeprom_write_string(E_INIT, "INIT");
	EEPROM.commit();
	//eeprom_serial_dump_xy(0, E_PASS + 20);
}

IPAddress SystemState::thermostatIP()
{
	return IPAddress(eepromUtil.eeprom_read_byte(E_IP0), eepromUtil.eeprom_read_byte(E_IP1), eepromUtil.eeprom_read_byte(E_IP2), eepromUtil.eeprom_read_byte(E_IP3));
}

void SystemState::setOperatingState(bool state) {
	ss.currentOperatingState = state;
	eepromUtil.eeprom_write_byte(E_LASTSYSTEMSTATE, state);
	EEPROM.commit();
}

void SystemState::debugOutParameters() {
	if (!WiFi.isConnected())
		return;
	delay(100);

	String jsonString = buildSettingsJson();
	if (WiFi.isConnected()) {
		wsd.textAll(jsonString.c_str());
	}
}

// Build the json with settings for setup page
String SystemState::buildSettingsJson() {
	String jsonString = Schedule.buildSettingsJson();		//Get the schedule settings
	// Add values in the object
	const size_t capacity = JSON_OBJECT_SIZE(33) + 320;
	DynamicJsonDocument root(capacity);
	deserializeJson(root, jsonString);

	root["defT"] = Defrost.defrostTemperature;
	root["defR"] = Defrost.defrostRuntime;
	root["defU"] = Defrost.defrostUpperTemperature;
	root["defMin"] = Defrost.defrostTimeBetween;
	root["heatT"] = ss.heatT;
	root["coolT"] = ss.coolT;
	root["hyst"] = ss.hysterisis;
	root["maxC"] = ss.maxCompTemperature;
	root["defD"] = Defrost.defrostDelay;
	root["targetT"] = ss.targetT;
	//root["roomT"] = ss.setRoomTemperature;
	root["user"] = ss.www_username;
	root["mode"] = oMode[ss.operatingMode];
	root["pass"] = ss.www_password;
	root["ip0"] = ss.thermostatIP()[0];
	root["ip1"] = ss.thermostatIP()[1];
	root["ip2"] = ss.thermostatIP()[2];
	root["ip3"] = ss.thermostatIP()[3];
	root["tstat"] = ss.useThermostat;
	root["thermostatID"] = ss.thermostatID;
	root["adcOffset"] = ss.adcOffset;
	root["nightTemperature"] = ss.nightTemperature;						//Nighttime reduction
	root["nightHour"] = ss.nightHour;
	root["dayHour"] = ss.dayHour;
	root["useWeather"] = ss.bUseWeather;
	root["runPumpWhenOff"] = ss.bRunPumpWhenOff;

	//Automatic function
	root["automatic"] = ss.automatic;
	root["startTime"] = ss.startTime;						
	root["stopTime"] = ss.stopTime;

	root["hostname"] = ss.hostName;
	root["topic"] = ss.topic;

	jsonString = "";
	serializeJson(root, jsonString);
	return jsonString;
}

void SystemState::saveSettings() {
	DEBUG_PRINTF(G("Settings:%s\n"), ss.JsonToSave.c_str());
	const size_t capacity = 800;
	DynamicJsonDocument root(capacity);
	deserializeJson(root, ss.JsonToSave);

	Defrost.defrostDelay = root["defD"];
	Defrost.defrostTemperature = root["defT"];
	Defrost.defrostRuntime = root["defR"];
	Defrost.defrostUpperTemperature = root["defU"];
	Defrost.defrostTimeBetween = root["defMin"];
	ss.heatT = root["heatT"];
	ss.coolT = root["coolT"];
	ss.hysterisis = root["hyst"];
	ss.maxCompTemperature = root["maxC"];
	ss.useThermostat = root["tstat"];
	ss.thermostatID = root["thermostatID"];
	ss.adcOffset = root["adcOffset"];
	ss.nightTemperature = root["nightTemperature"];						//Nighttime reduction
	ss.nightHour = root["nightHour"];
	ss.dayHour = root["dayHour"];
	ss.bUseWeather = root["useWeather"];
	ss.bRunPumpWhenOff = root["runPumpWhenOff"];

	ss.automatic = root["automatic"];
	ss.startTime = root["startTime"];
	ss.stopTime = root["stopTime"];
	snprintf(ss.topic, sizeof(ss.topic), root["topic"]);
	snprintf(ss.topicr, sizeof(ss.topicr), "%sr", ss.topic);
	const char* hostname = root["hostname"]; 
	snprintf(ss.hostName, sizeof(ss.hostName), hostname);

	String usr = root["user"];
	String pwd = root["pass"];
	eepromUtil.eeprom_write_string(E_USER, usr.c_str());
	eepromUtil.eeprom_write_string(E_PASS, pwd.c_str());
	eepromUtil.eeprom_write_byte(E_DEFROSTTEMP, Defrost.defrostTemperature + DEFROST_OFFSET);			//Use offset to avoid negative numbers
	eepromUtil.eeprom_write_word(E_DEFROSTTIME, Defrost.defrostRuntime);
	eepromUtil.eeprom_write_byte(E_DEFROSTMAXTEMP, Defrost.defrostUpperTemperature);
	eepromUtil.eeprom_write_byte(E_TIMEBBETWEENDEFROSTS, Defrost.defrostTimeBetween);
	eepromUtil.eeprom_write_byte(E_HEATTEMP, ss.heatT);
	eepromUtil.eeprom_write_byte(E_COOLTEMP, ss.coolT);
	eepromUtil.eeprom_write_byte(E_HYSTERISIS, ss.hysterisis);
	eepromUtil.eeprom_write_byte(E_MAXCOMPTEMP, ss.maxCompTemperature);
	eepromUtil.eeprom_write_word(E_DEFROSTDELAY, Defrost.defrostDelay);
	eepromUtil.eeprom_write_byte(E_IP0, root["ip0"]);
	eepromUtil.eeprom_write_byte(E_IP1, root["ip1"]);
	eepromUtil.eeprom_write_byte(E_IP2, root["ip2"]);
	eepromUtil.eeprom_write_byte(E_IP3, root["ip3"]);
	eepromUtil.eeprom_write_byte(E_USETHERMOSTAT, ss.useThermostat);
	eepromUtil.eeprom_write_int(E_THERMOSTATID, ss.thermostatID);
	eepromUtil.eeprom_write_byte(E_ADCOFFSET, ss.adcOffset);
	eepromUtil.eeprom_write_byte(E_NIGHTREDUCTION, ss.nightTemperature);
	eepromUtil.eeprom_write_byte(E_NIGHTHOUR, ss.nightHour);
	eepromUtil.eeprom_write_byte(E_DAYHOUR, ss.dayHour);
	eepromUtil.eeprom_write_byte(E_USEWEATHER, ss.bUseWeather);
	eepromUtil.eeprom_write_byte(E_RUNPUMPWHENOFF, ss.bRunPumpWhenOff);

	eepromUtil.eeprom_write_byte(E_AUTOMATIC, ss.automatic);
	eepromUtil.eeprom_write_byte(E_STARTTIME, ss.startTime);
	eepromUtil.eeprom_write_byte(E_STOPTIME, ss.stopTime);
	eepromUtil.eeprom_write_string(E_HOSTNAME, ss.hostName);
	eepromUtil.eeprom_write_string(E_TOPIC, ss.topic);

	EEPROM.commit();
	getTargetTemperature();
	DEBUG_PRINTLN(G("settings saved"));
	eepromUtil.eeprom_read_string(E_HOSTNAME, ss.hostName, sizeof(ss.hostName));
	Serial.printf("(Saved)Hostname:%s\n", ss.hostName);
}

//Get target temperature
void SystemState::getTargetTemperature() {

	//Get the average temperature between now and the next forecast and set the temperature to that
	if (ss.bUseWeather) {
		//ss.targetT = (Weather.forecast[0].desiredTemperature + Weather.forecast[1].desiredTemperature) / 2;

		float t1 = (Weather.forecast[0].desiredTemperature + Weather.forecast[0].desiredFeelLikeTemperature) / 2;
		float t2 = (Weather.forecast[1].desiredTemperature + Weather.forecast[1].desiredFeelLikeTemperature) / 2;

		ss.targetT = (t1 + t2) / 2;
		Serial.printf("T1: %f\n", t1);
		Serial.printf("T2: %f\n", t2);
		Serial.printf("targetT %d\n", ss.targetT);
	}
	else {
		if (ss.operatingMode == HEAT)
			ss.targetT = ss.heatT;
		else
			ss.targetT = ss.coolT;
	}
}

// Create a string with the json for both the browser and the app
String SystemState::getSystemStatus(bool bWeb) {
	char relays[200];
	char tempHum[30];
	char temp[10];

	// Add values in the object
	const size_t capacity = 1500;
	DynamicJsonDocument root(capacity);
	root["heatpump"] = 1;
	root["hpName"] = ss.hostName;
	root["timestamp"] = ntp.getLocalTime();
	root["actual"] = TEMP_WATER;
	root["compressor"] = TEMP_COMPRESSOR;
	root["evaporator"] = TEMP_EVAPORATOR;
	root["midevap"] = TEMP_EVAPMID;
	root["external"] = TEMP_EXTERNAL;
	if (ss.bUseWeather)
		snprintf(temp, sizeof(temp), "%d (%s)", ss.targetT, ntp.isTimeDaytime(Weather.forecast[0].forecastTime) ? "D" : "N");
	else
		snprintf(temp, sizeof(temp), "%d", ss.targetT);

	root["targetT"] = temp;
	if (bWeb) {
		snprintf(relays, sizeof(relays), "C:%s F:%s V:%s P:%s", oState[ss.compressorState], oState[ss.fanState], oState[ss.valveState], oState[ss.pumpState]);
		root["relays"] = relays;
		root["mode"] = oMode[ss.operatingMode];
	}
	snprintf(relays, sizeof(relays), "%d%d%d%d", ss.compressorState, ss.fanState, ss.valveState, ss.pumpState);
	root["relaysb"] = relays;
	root["bMmode"] = (int8_t)ss.operatingMode;
	root["defrost"] = Defrost.defrostFlag;
	root["defrostStep"] = Defrost.defrostStep;
	//root["defrostRunTime"] = Defrost.defrostTimer;
	root["lastDefrost"] = Defrost.lastDefrostTime;
	if (ss.overheat) {
		root["onoff"] = "Off OT";
	}
	else
	{
		if(ss.automatic)
			root["onoff"] = oState[2];
		else
			root["onoff"] = oState[ss.masterOnOff];
	}
	root["bOnoff"] = ss.masterOnOff;
	root["opState"] = ss.currentOperatingState;
	root["thermostatT"] = ss.thermostatTemperature;
	root["setRoomT"] = ss.setRoomTemperature;
	root["useWeather"] = ss.bUseWeather;
	root["lastWeather"] = ntp.printTime(Weather.forecast[0].forecastTime);
	root["startTime"] = ss.startTime;
	root["stopTime"] = ss.stopTime;
	root["auto"] = ss.automatic;
	root["compressorState"] = ss.compressorState;

	String jsonString;
	serializeJson(root, jsonString);
	return jsonString;
}

void SystemState::sendSystemStatus() {
	if (!WiFi.isConnected())
		return;

	Weather.sendWeatherJson();					//Send weather information
	//Send json with system status to web browser
	ws.textAll(getSystemStatus(true));
	//..and to MQTT
	String jsonString = getSystemStatus(false);
	mqttClient.publish(ss.topicr, jsonString.c_str(), jsonString.length(), 0, false);
	//mqttClient.publish(ss.topicr, 0, false, ss.getSystemStatus(false));
}

void SystemState::sendWsMessage(String message, String key) {
	String jsonString;
	// Add values in the object
	const size_t capacity = JSON_OBJECT_SIZE(4) + 250;
	DynamicJsonDocument root(capacity);
	root[key] = message;
	serializeJson(root, jsonString);
	ws.textAll(jsonString);
}

// Turn on/off relays and preserve state flag
void SystemState::setRelay(int relay, bool setting)
{
	char temp[200];
	byte data = 0xff;

	last_loop = millis();							//Timer for watchdog
	switch (relay) {
	case COMPRESSOR:
		if (!ss.overheat) {
			if (ss.compressorState != setting) {
				ss.compressorState = setting;
			}
		}
		break;
	case VALVE:
		if (ss.valveState != setting) {
			ss.valveState = setting;
		}
		break;
	case FAN:
		if (ss.fanState != setting) {
			ss.fanState = setting;
		}
		break;
	case PUMP:
		if (ss.pumpState != setting) {
			ss.pumpState = setting;
		}
		digitalWrite(PUMPGPIO, !ss.pumpState);
		digitalWrite(PUMPGPIO1, ss.pumpState);
		break;
	}

	//Build the value to send to the PCF8574
	if ((ss.compressorState == ON)&&(ss.masterOnOff == ON))
		data &= COMPRESSOR;
	if (ss.valveState == ON)
		data &= VALVE;
	if (ss.fanState == ON)
		data &= FAN;
	//if (ss.pumpState == ON)		//Pump is on board relay, not on expander
	//	data &= PUMP;

	DEBUG_PRINTF("Relays %d\n", data);
	expanderWrite(data);

	//Send update if we are really changing a state
	if (relay != 0) {
		snprintf(temp, sizeof(temp), "C:%s F:%s V:%s P:%s", oState[ss.compressorState], oState[ss.fanState], oState[ss.valveState], oState[ss.pumpState]);
		ss.sendWsMessage(temp, "relays");
	}
}

//PCF8574 I2C Expander
void SystemState::expanderWrite(byte data)
{
	Wire.beginTransmission(PCF8574);
	Wire.write(data);
	Wire.endTransmission();
}

/////////////////////////////////////////////  Network   ////////////////////////////////////////////////////
//Get the network settings data
String SystemState::buildNetSettingsJson() {
	char add[16];
	char buf[10];
	String jsonString;

	// Add values in the object
	const size_t capacity = JSON_OBJECT_SIZE(9) + 140;
	DynamicJsonDocument root(capacity);

	eepromUtil.eeprom_read_string(E_SSID, ss.ssid, sizeof(ssid));
	eepromUtil.eeprom_read_string(E_WIFIPASS, ss.wifipass, sizeof(wifipass));
	root["ssid"] = ss.ssid;
	root["wifipass"] = ss.wifipass;
	root["hostname"] = ss.hostName;
	eepromUtil.eeprom_read_string(E_IPADD, buf, 5, true);
	sprintf(add, "%d.%d.%d.%d", buf[0], buf[1], buf[2], buf[3]);
	root["ipadd"] = add;
	eepromUtil.eeprom_read_string(E_MASK, buf, 5, true);
	sprintf(add, "%d.%d.%d.%d", buf[0], buf[1], buf[2], buf[3]);
	root["mask"] = add;
	eepromUtil.eeprom_read_string(E_GATEWAY, buf, 5, true);
	sprintf(add, "%d.%d.%d.%d", buf[0], buf[1], buf[2], buf[3]);
	root["gateway"] = add;
	eepromUtil.eeprom_read_string(E_DNS1, buf, 5, true);
	sprintf(add, "%d.%d.%d.%d", buf[0], buf[1], buf[2], buf[3]);
	root["dns1"] = add;
	eepromUtil.eeprom_read_string(E_DNS2, buf, 5, true);
	sprintf(add, "%d.%d.%d.%d", buf[0], buf[1], buf[2], buf[3]);
	root["dns2"] = add;
	root["dhcp"] = eepromUtil.eeprom_read_byte(E_DHCP);
	serializeJson(root, jsonString);
	return jsonString;
}


void parseBytes(const char* str, char sep, byte* bytes, int maxBytes, int base) {
	for (int i = 0; i < maxBytes; i++) {
		bytes[i] = strtoul(str, NULL, base);  // Convert byte
		str = strchr(str, sep);               // Find next separator
		if (str == NULL || *str == '\0') {
			break;                            // No more separators, exit
		}
		str++;                                // Point to next character after separator
	}
}


void SystemState::saveNetSettings() {
	DEBUG_PRINTF(G("saveNet: %s\n"), ss.JsonToSave.c_str());

	const char* ipStr;
	byte ip[4];
	const size_t capacity = JSON_OBJECT_SIZE(9) + 180;
	DynamicJsonDocument root(capacity);
	deserializeJson(root, ss.JsonToSave);

	snprintf(ssid, sizeof(ssid), root["ssid"]);
	Serial.printf("SSID(0):%s\n", ss.ssid);
	eepromUtil.eeprom_write_string(E_SSID, ss.ssid);
	eepromUtil.eeprom_write_string(E_WIFIPASS, root["wifipass"]);
	eepromUtil.eeprom_write_byte(E_DHCP, root["dhcp"]);
	ipStr = root["ipadd"];
	parseBytes(ipStr, '.', ip, 4, 10);
	eepromUtil.eeprom_write_string(E_IPADD, (const char*)ip, 4);
	ipStr = root["gateway"];
	parseBytes(ipStr, '.', ip, 4, 10);
	eepromUtil.eeprom_write_string(E_GATEWAY, (const char*)ip, 4);
	ipStr = root["dns1"];
	parseBytes(ipStr, '.', ip, 4, 10);
	eepromUtil.eeprom_write_string(E_DNS1, (const char*)ip, 4);
	ipStr = root["dns2"];
	parseBytes(ipStr, '.', ip, 4, 10);
	eepromUtil.eeprom_write_string(E_DNS2, (const char*)ip, 4);
	ipStr = root["mask"];
	parseBytes(ipStr, '.', ip, 4, 10);
	eepromUtil.eeprom_write_string(E_MASK, (const char*)ip, 4);
	eepromUtil.eeprom_write_string(E_HOSTNAME, root["hostname"]);
	ss.JsonToSave = "";

	EEPROM.commit();

	Serial.printf("SSID(1):%s\n", ss.ssid);
	eepromUtil.eeprom_read_string(E_SSID, ss.ssid, sizeof(ss.ssid));
	Serial.printf("SSID(2):%s\n", ss.ssid);

}

//If current time is between start and stop times then return true
bool SystemState::getTimeBetween(int hourNow, int start, int stop) {
	bool timeTrue = false;

	if ((hourNow >= start) || (hourNow < stop))
		timeTrue = true;

	return timeTrue;
}

//Set and save automatic setting
void SystemState::setAuto(bool setting) {
	ss.automatic = setting;
	eepromUtil.eeprom_write_int(E_AUTOMATIC, ss.automatic);
	EEPROM.commit();
}

void SystemState::powerOnOff() {
	DEBUG_PRINTF("Set power: %s\n", oState[ss.masterOnOff]);
	if (ss.masterOnOff == OFF)
	{
		DEBUG_PRINTLN(G("Turning OFF"));
		if (Defrost.defrostFlag) {
			tickerDefrost.detach();
			Defrost.defrostFlag = false;
			Defrost.defrostStep = 0;
		}
		ss.setRelay(COMPRESSOR, OFF);
		ss.setRelay(FAN, OFF);
		ss.bValveOff = true;//  setRelay(VALVE, OFF);
		ss.setOperatingState(OFF);
		ss.startupDelay = true;
		ss.millisNow = millis();
		DEBUG_PRINTF("Millis:%ld\n", ss.millisNow);
	}
	Defrost.lastDefrost = millis();					//Set so defrost isn't run on startup as the evap can get very cold on startup before stablizing 
	ss.justStarted = true;							//Signal on so ignore hysterisis
	eepromUtil.eeprom_write_byte(E_ONOFF, ss.masterOnOff);
	EEPROM.commit();
	ss.bMain = true;								//Force a run of main logic
}

void SystemState::turnOff() {
	if (ss.currentOperatingState == ON) {
		DEBUG_PRINTLN(G("Turning OFF"));
		ss.setRelay(COMPRESSOR, OFF);
		ss.setRelay(FAN, OFF);
		ss.bValveOff = true;//  setRelay(VALVE, OFF);
		ss.setOperatingState(OFF);
		if (Defrost.defrostFlag) {
			tickerDefrost.detach();
			Defrost.defrostFlag = false;
		}
		ss.refreshData = true;
	}
}

SystemState ss;
